{-# LANGUAGE PackageImports #-}
-- | The global state of the client - leases, cache, etc.
module Skynet.Client.State(
  HandleID, FMap,
  ClientState(..),
  blankClientState)
where

import Skynet.Protocol

import Control.Applicative
import Control.Concurrent.STM
import Network.Hermes.MChan
import qualified Data.Map as M
import Data.Map(Map)
import Data.Word(Word64)
import System.Fuse(FileStat(..))

type HandleID = Word64

-- data HandleData = HandleData LeaseID

type FMap a = Map FilePath a

data ClientState = ClientState {
  defaultCoherency :: Coherency
  ,leasePath :: TVar (Map FilePath Lease)
  ,leaseHandle :: TVar (Map HandleID Lease)
  ,nextHandle :: TVar Word64
   -- Caches/buffers (the same structures are used for both)
  ,dirCache :: TVar (FMap [FilePath])
  ,fstatCache :: TVar (FMap FileStat)
  ,linkCache :: TVar (FMap FilePath)
   -- Queue of functions to call to flush various leases
  ,sendQueue :: MChan LeaseID (IO (),Int) -- ^ (Function, buffer size)
  ,sendQueueSize :: TVar Int -- ^ An abstract notion of buffer size
  }

tmap = newTVarIO M.empty

blankClientState :: IO ClientState
blankClientState = ClientState POSIX <$> tmap <*> tmap <*> newTVarIO 1 <*> tmap <*> tmap <*>
                   tmap <*>
                   newMChanIO <*> newTVarIO 0
