{-# LANGUAGE TemplateHaskell #-}
module Skynet.Protocol.Cereal where

import Skynet.Protocol.Base
import Data.Derive.Serialize
import Data.DeriveTH
import Data.Serialize

import Foreign.C.Error(Errno(..))
import System.Fuse(FileStat(..), EntryType(..))
import Foreign.C.Types
import System.Posix.Types
import Data.Word(Word32, Word64)
import Data.Int(Int64)
import Control.Applicative

-- Hermes serializers
derives [makeSerialize] [''ResolverMessage,''URIType,''Address, ''MessageWrapper, ''Message,
                         ''Lease, ''Coherency, ''LeaseMode, ''URI, ''Errno,
                         ''LeaseType, ''FileStat, ''EntryType, ''ResolverTag]

-- The below instances cannot be derived, due to not exporting
-- internals. Converting to integers and back works, however.

instance Serialize CInt where
  get = (fromIntegral :: Word32 -> CInt) <$> get
  put = put . (fromIntegral :: CInt -> Word32)

instance Serialize CMode where
  get = (fromIntegral :: Word32 -> CMode) <$> get
  put = put . (fromIntegral :: CMode -> Word32)

instance Serialize CTime where
  get = (fromIntegral :: Int64 -> CTime) <$> get
  put = put . (truncate . toRational :: CTime -> Int64)

instance Serialize COff where
  get = (fromIntegral :: Int64 -> COff) <$> get
  put = put . (fromIntegral :: COff -> Int64)

instance Serialize CDev where
  get = (fromIntegral :: Word64 -> CDev) <$> get
  put = put . (truncate . toRational :: CDev -> Word64)

instance Serialize CNlink where
  get = (fromIntegral :: Word64 -> CNlink) <$> get
  put = put . (fromIntegral :: CNlink -> Word64)

instance Serialize CGid where
  get = (fromIntegral :: Word32 -> CGid) <$> get
  put = put . (fromIntegral :: CGid -> Word32)

instance Serialize CUid where
  get = (fromIntegral :: Word32 -> CUid) <$> get
  put = put . (fromIntegral :: CUid -> Word32)
