{-# LANGUAGE ViewPatterns, TupleSections, PackageImports, OverlappingInstances,
             FlexibleInstances, UndecidableInstances, ScopedTypeVariables #-}
module Skynet.Util(
  modifyTVar,chainMap,log,loopUntil,Prompt,prompt,askLine,Priority(..),throwM,stripRoot,stripRoot'
  ) where

import Control.Concurrent.STM
import qualified Data.Map as M
import Data.Map(Map)
import qualified Network.Hermes.Misc as Misc
import System.Log.Logger
import "mtl" Control.Monad.Trans
import Control.Monad
import Prelude hiding(log)
import System.IO
import System.Console.Haskeline
import Data.Maybe
import Control.Applicative

modifyTVar :: TVar a -> (a -> a) -> STM ()
modifyTVar = Misc.modifyTVar

chainMap :: (Ord a, Ord b) => Map a b -> Map b c -> Map a (b,c)
chainMap left right = M.mapMaybe (\b -> (b,) `fmap` M.lookup b right) left

log :: (MonadIO m, Show s) => Priority -> String -> String -> s -> m ()
log prio source msg s = liftIO $ logM source prio (source ++ ": " ++ msg ++ show s)

loopUntil :: (a -> Bool) -> IO a -> IO a
loopUntil cond act = do
  val <- act
  if cond val then return val else loopUntil cond act

-- Prompts

class Prompt a where
  prompt :: String -> IO a

askLine :: String -> IO String
askLine = fmap fromJust . runInputT defaultSettings . getInputLine

instance Prompt Bool where
  prompt s = do
    v <- loopUntil (liftM2 (||) ("y" ==) ("n" ==)) (askLine (s ++ " (y/n) "))
    case () of
      () | v == "y" -> return True
      otherwise -> return False

instance forall a. (Read a) => Prompt a where
  prompt s = do
    v <- loopUntil (not . null . (reads :: String -> [(a,String)])) (askLine s)
    return . fst . head $ reads v

throwM :: (MonadIO m, Exception e) => e -> m a
throwM = liftIO . throwIO


-- | Strip the root off a path.. assuming it has one
stripRoot :: [FilePath] -> [FilePath]
stripRoot (('/':[]):path) = path
stripRoot path = path

stripRoot' :: FilePath -> FilePath
stripRoot' ('/':path) = path
stripRoot' path = path
