{-# LANGUAGE ScopedTypeVariables, TemplateHaskell, GeneralizedNewtypeDeriving, DeriveDataTypeable #-}
module Tests where

import Network.Hermes

import Data.IORef
import Control.Monad
import Control.Concurrent
import qualified Data.Map as M
import Data.Typeable
import Data.Serialize(Serialize)
import qualified Data.ByteString as B
import Control.Applicative
import System.IO

import System.Random
import Test.QuickCheck

import Test.HUnit

data TestMode = Server | Client

newtype Quit = Quit ()
             deriving(Serialize,Typeable)

clientA = IPv4 "localhost" 9999
serverA = IPv4 "localhost" 9998

testDepth :: Num a => a
testDepth = 20


ints = [1..testDepth::Int]

coreServer ctx = do
  forM_ ints $ \i -> do
    (_,i') <- recv ctx
    i @=? i'

coreClient ctx uuid = do
  forM_ ints $ \i -> do 
    send ctx uuid i
    -- threadDelay 100000

rpcServer ctx = do
  registerCallback ctx "succ" (return . (succ :: Int -> Int))

rpcClient ctx uuid = do
  -- We need to pause here so the server gets a chance to register its
  -- callback
  threadDelay 100000
  ourData <- replicateM testDepth randomIO
  forM_ ourData $ \int -> do
    Just int' <- call ctx uuid "succ" (int :: Int)
    (succ int) @=? int'
    -- threadDelay 100000

runTests Server ctx = do
  setDebug DEBUG
  startListener ctx serverA Nothing
  rpcServer ctx -- Set this up for later
  coreServer ctx
  (_,Quit ()) <- recv ctx
  return ()

runTests Client ctx = do
  -- setDebug DEBUG
  startListener ctx clientA Nothing
  serverHermesID <- connect ctx serverA
  coreClient ctx serverHermesID
  rpcClient ctx serverHermesID
  send ctx serverHermesID (Quit ())
